@@ -1,7 +1,7 @@
 from .version import __version__
 
 import os
-motd = '''      ___           ___           ___           ___
+motd = r'''      ___           ___           ___           ___
      /__/\         /__/\         /__/\         /  /\      Version {:<8}
      \  \:\       |  |::\       |  |::\       /  /::\ 
       \  \:\      |  |:|:\      |  |:|:\     /  /:/\:\    An open source
@@ -14,19 +14,25 @@ motd = '''      ___           ___           ___           ___
      \__\/         \__\/         \__\/         \__\/      Phillip Isola's lab '''.format(__version__)
 
 from . import scripting
-from .lib import material
-from .lib.rating import OpenSkillRating
+from .lib import material, spawn
 from .overlay import Overlay, OverlayRegistry
 from .io import action
 from .io.stimulus import Serialized
 from .io.action import Action
 from .core import config, agent
 from .core.agent import Agent
-from .core.env import Env
+from .core.env import Env, Replay
+from . import scripting, emulation, integrations
 from .systems.achievement import Task
 from .core.terrain import MapGenerator, Terrain
 
-__all__ = ['Env', 'config', 'scripting', 'agent', 'Agent', 'MapGenerator', 'Terrain',
-        'Serialized', 'action', 'Action', 'scripting', 'material',
-        'Task', 'Overlay', 'OverlayRegistry']
+__all__ = ['Env', 'config', 'scripting', 'emulation', 'integrations', 'agent', 'Agent', 'MapGenerator', 'Terrain',
+        'Serialized', 'action', 'Action', 'scripting', 'material', 'spawn',
+        'Task', 'Overlay', 'OverlayRegistry', 'Replay']
 
+try:
+    import openskill
+    from .lib.rating import OpenSkillRating
+    __all__.append('OpenSkillRating')
+except:
+    print('Warning: OpenSkill not installed. Ignore if you do not need this feature')
