@@ -1,10 +1,13 @@
 from pdb import set_trace as T
 
-from nmmo.lib.colors import Palette
+from nmmo.lib import colors
 
 class Agent:
     scripted = False
-    name     = 'Neural_'
+    policy   = 'Neural'
+
+    color    = colors.Neon.CYAN
+    pop      = 0
 
     def __init__(self, config, idx):
        '''Base class for agents
@@ -15,6 +18,7 @@ class Agent:
        '''
        self.config = config
        self.iden   = idx
+       self.pop    = Agent.pop
 
     def __call__(self, obs):
        '''Used by scripted agents to compute actions. Override in subclasses.
