@@ -1,15 +1,22 @@
 from pdb import set_trace as T
 import numpy as np
 
+from ordered_set import OrderedSet
 from collections import defaultdict
 from collections.abc import Mapping
 from typing import Dict, Callable
 
 import nmmo
 from nmmo import core, infrastructure
+from nmmo.systems.exchange import Exchange
+from nmmo.systems import combat
 from nmmo.entity.npc import NPC
 from nmmo.entity import Player
-from nmmo.lib import colors
+from nmmo.systems.item import Item
+
+from nmmo.io.action import Action
+from nmmo.lib import colors, spawn, log
+
 
 def prioritized(entities: Dict, merged: Dict):
    '''Sort actions into merged according to priority'''
@@ -18,6 +25,7 @@ def prioritized(entities: Dict, merged: Dict):
          merged[atn.priority].append((idx, (atn, args.values())))
    return merged
 
+
 class EntityGroup(Mapping):
    def __init__(self, config, realm):
       self.dataframe = realm.dataframe
@@ -53,7 +61,6 @@ class EntityGroup(Mapping):
       for entID, ent in self.entities.items():
          self.dataframe.remove(nmmo.Serialized.Entity, entID, ent.pos)
 
-      self.spawned  = False
       self.entities = {}
       self.dead     = {}
 
@@ -89,26 +96,36 @@ class EntityGroup(Mapping):
       for entID, entity in self.entities.items():
          entity.update(self.realm, actions)
 
+
 class NPCManager(EntityGroup):
    def __init__(self, config, realm):
       super().__init__(config, realm)
       self.realm   = realm
 
+      self.spawn_dangers = []
+
    def reset(self):
       super().reset()
       self.idx     = -1
 
    def spawn(self):
-      if not self.config.game_system_enabled('NPC'):
+      config = self.config
+
+      if not config.NPC_SYSTEM_ENABLED:
          return
 
-      for _ in range(self.config.NPC_SPAWN_ATTEMPTS):
-         if len(self.entities) >= self.config.NMOB:
+      for _ in range(config.NPC_SPAWN_ATTEMPTS):
+         if len(self.entities) >= config.NPC_N:
             break
 
-         center = self.config.TERRAIN_CENTER
-         border = self.config.TERRAIN_BORDER
-         r, c   = np.random.randint(border, center+border, 2).tolist()
+         if self.spawn_dangers:
+            danger = self.spawn_dangers[-1]
+            r, c   = combat.spawn(config, danger)
+         else:
+            center = config.MAP_CENTER
+            border = self.config.MAP_BORDER
+            r, c   = np.random.randint(border, center+border, 2).tolist()
+
          if self.realm.map.tiles[r, c].occupied:
             continue
 
@@ -117,6 +134,13 @@ class NPCManager(EntityGroup):
             super().spawn(npc)
             self.idx -= 1
 
+         if self.spawn_dangers:
+            self.spawn_dangers.pop()
+
+   def cull(self):
+       for entity in super().cull().values():
+           self.spawn_dangers.append(entity.spawn_danger)
+
    def actions(self, realm):
       actions = {}
       for idx, entity in self.entities.items():
@@ -126,42 +150,49 @@ class NPCManager(EntityGroup):
 class PlayerManager(EntityGroup):
    def __init__(self, config, realm):
       super().__init__(config, realm)
-
-      self.loader  = config.AGENT_LOADER
       self.palette = colors.Palette()
+      self.loader  = config.PLAYER_LOADER
       self.realm   = realm
 
    def reset(self):
       super().reset()
       self.agents  = self.loader(self.config)
-      self.idx     = 1
+      self.spawned = OrderedSet()
 
-   def spawnIndividual(self, r, c):
+   def spawnIndividual(self, r, c, idx):
       pop, agent = next(self.agents)
-      agent      = agent(self.config, self.idx)
+      agent      = agent(self.config, idx)
       player     = Player(self.realm, (r, c), agent, self.palette.color(pop), pop)
       super().spawn(player)
-      self.idx   += 1
 
    def spawn(self):
-      if self.config.SPAWN == self.config.SPAWN_CONCURRENT:
-         if self.spawned:
-            return 
-
-         self.spawned = True
+      #TODO: remove hard check against fixed function
+      if self.config.PLAYER_SPAWN_FUNCTION == spawn.spawn_concurrent:
          idx = 0
-         for r, c in self.config.SPAWN():
+         for r, c in self.config.PLAYER_SPAWN_FUNCTION(self.config):
             idx += 1
-            assert not self.realm.map.tiles[r, c].occupied
-            self.spawnIndividual(r, c)
+
+            if idx in self.entities:
+                continue
+
+            if idx in self.spawned and not self.config.RESPAWN:
+                continue
+
+            self.spawned.add(idx)
+            
+            if self.realm.map.tiles[r, c].occupied:
+                continue
+
+            self.spawnIndividual(r, c, idx)
+
          return
           
       #MMO-style spawning
       for _ in range(self.config.PLAYER_SPAWN_ATTEMPTS):
-         if len(self.entities) >= self.config.NENT:
+         if len(self.entities) >= self.config.PLAYER_N:
             break
 
-         r, c   = self.config.SPAWN()
+         r, c   = self.config.PLAYER_SPAWN_FUNCTION(self.config)
          if self.realm.map.tiles[r, c].occupied:
             continue
 
@@ -174,35 +205,56 @@ class Realm:
    '''Top-level world object'''
    def __init__(self, config):
       self.config   = config
+      Action.hook(config)
 
-      #Generate maps if they do not exist
+      # Generate maps if they do not exist
       config.MAP_GENERATOR(config).generate_all_maps()
 
-      #Load the world file
-      self.dataframe = infrastructure.Dataframe(config)
+      # Load the world file
+      self.dataframe = infrastructure.Dataframe(self)
       self.map       = core.Map(config, self)
 
-      #Entity handlers
+      # Entity handlers
       self.players  = PlayerManager(config, self)
       self.npcs     = NPCManager(config, self)
 
+      # Global item exchange
+      self.exchange = Exchange()
+
+      # Global item registry
+      self.items    = {}
+
+      # Initialize actions
+      nmmo.Action.init(config)
+
    def reset(self, idx):
       '''Reset the environment and load the specified map
 
       Args:
          idx: Map index to load
       ''' 
+      Item.INSTANCE_ID = 0
+      self.quill = log.Quill(self.config)
       self.map.reset(self, idx)
       self.players.reset()
       self.npcs.reset()
       self.tick = 0
- 
+
+      # Global item exchange
+      self.exchange = Exchange()
+
+      # Global item registry
+      self.items    = {}
+
    def packet(self):
       '''Client packet'''
       return {'environment': self.map.repr,
+              'border': self.config.MAP_BORDER,
+              'size': self.config.MAP_SIZE,
               'resource': self.map.packet,
               'player': self.players.packet,
-              'npc': self.npcs.packet}
+              'npc': self.npcs.packet,
+              'market': self.exchange.packet}
 
    @property
    def population(self):
@@ -234,6 +286,11 @@ class Realm:
 
       #Execute actions
       for priority in sorted(merged):
+         # Buy/sell priority
+         entID, (atn, args) = merged[priority][0]
+         if atn in (nmmo.action.Buy, nmmo.action.Sell):
+            merged[priority] = sorted(merged[priority], key=lambda x: x[0]) 
+
          for entID, (atn, args) in merged[priority]:
             ent = self.entity(entID)
             atn.call(self, ent, *args)
